import os
import time
import psutil
import urllib
import sqlite3
import tarfile
import ipaddress
import ConfigParser

try:
    import httpbl
except:
    pass

try:
    import geoip2.database
except:
    pass
    
def get_process_name(pid):
    try:
        if pid==None:
            return 'None'
        else:
            p_name=psutil.Process(pid).name()  
            return p_name
    except:        
        return 'None'
        
def project_honey_pot(ip,key):
    bl = httpbl.HttpBL(key)
    response = bl.query(ip)
    return response['threat_score'],response['days_since_last_activity']

def validIP(address):
    parts = address.split(".")
    if len(parts) != 4:
        return False
    for item in parts:
        if not 0 <= int(item) <= 255:
            return False
    return True

def extract_ip(x,ip=True):
    try:
        if validIP(x[0])==True and ip==True:
            return x[0]
        elif validIP(x[0])==True and ip==False:
            return x[1]
        else:
            return None
    except:        
        return None


def geoip2_location(directory,ip):
    try:
        reader = geoip2.database.Reader(directory+"GeioLite2_City/GeoLite2-City.mmdb")
        response = reader.city(ip)
        country=str(response.country.name)
        city=str(response.city.name)
        reader.close()
        return ','.join([city,country])
    except:
        print("Error")
        return None


def geolite2_download(directory):
    geoip_path=directory+"GeioLite2_City/"
    web="https://geolite.maxmind.com/download/geoip/database/GeoLite2-City.tar.gz"
    try:
        if not os.path.isfile(geoip_path+"GeoLite2-City.mmdb"):
            if not os.path.exists(geoip_path):
                os.makedirs(geoip_path)
            print "Downloading GeoIP location file from maxmind.com"
            print "Please wait."
            urllib.urlretrieve(web,geoip_path+"GeoLite2-City.tar.gz")
            print "Download complete"            
            print "Extracting files from  GeoLite2-City.tar.gz"
            tar = tarfile.open(geoip_path+"GeoLite2-City.tar.gz", "r:gz")
            #tar.extractall(geoip_path=geoip_path)
            s= tar.getnames()
            s=[s for s in s if "GeoLite2-City.mmdb" in s]
            tar.extract(s[0],geoip_path)
            tar.close()
            print "Moving files to Geolite2-City"
            for i in os.listdir(geoip_path):    
                if os.path.isdir(geoip_path+i):                    
                    os.rename(geoip_path+i+"/"+"GeoLite2-City.mmdb",\
                    geoip_path+"GeoLite2-City.mmdb")
                    print "Removing tar file"
                    os.remove(geoip_path+"GeoLite2-City.tar.gz")
                    os.rmdir(geoip_path+i)

            print "Download complete"
        else:
            print "GeoIP location loaded"
    except:
        print "Error while downloading GeoIP location file from maxmind"
        print "Please check your network connection"
    


def config_para(directory,configfile_name):
    VERSION="0.1.7.4"    
    # Check if there a directory exists or not
    if not os.path.exists(directory):
        os.makedirs(directory)
    
    # Check if there is already a configurtion file
    Config = ConfigParser.ConfigParser()
    if not os.path.isfile(directory+configfile_name):
        # Create the configuration file as it doesn't exist yet
        cfgfile = open(directory+configfile_name, 'w')
        
        # Add content to the file       
        Config.add_section('pyort')
        Config.set('pyort', 'db_path', directory)
        Config.set('pyort', 'db_name','pyort.db')
        Config.set('pyort', 'interval',10)
        Config.set('pyort', 'kind',"all")
        Config.set('pyort', 'geo_ip','')
        Config.set('pyort', 'project_honey_pot_key','')
        Config.set('pyort', 'threat_update_count',1000)
        Config.set('pyort', 'version',VERSION)
        Config.write(cfgfile)
        cfgfile.close()
  
    try:
        Config.read(directory+configfile_name)
        Config.set('pyort', 'version',VERSION)
        with open(directory+configfile_name, 'wb') as configfile:
            Config.write(configfile)
        db_path= Config.get('pyort','db_path')
        db_name= Config.get('pyort','db_name')
        slp= Config.get('pyort','interval')
        kd= Config.get('pyort','kind')
        geo_ip=Config.get('pyort', 'geo_ip')
        hp_key=Config.get('pyort', 'project_honey_pot_key')
        threat_update=Config.get('pyort', 'threat_update_count')
        version=Config.get('pyort', 'version')        
    except Exception as e:
        print e.__doc__
        print e.message
        print("Got some error in [config.ini], Deleting old one and creating new [config.ini].")        
        os.remove(directory+configfile_name)
        config_para(directory,configfile_name)  
    
    return db_path,db_name,slp,kd,hp_key,threat_update,version,geo_ip
    

def sqlite_conn(db_path,db_name):
    try:
        db_conn= sqlite3.connect(db_path+db_name)        
        db_conn.execute('''CREATE TABLE IF NOT EXISTS pyort
                        (id INTEGER PRIMARY KEY,
                         first_time  DATETIME DEFAULT CURRENT_TIMESTAMP,
                         last_time  DATETIME DEFAULT CURRENT_TIMESTAMP,
                         fd TEXT NOT NULL,
                         family TEXT NOT NULL,
                         conn_type TEXT NOT NULL,
                         local_ip TEXT NOT NULL,
                         local_port TEXT NOT NULL,
                         remote_ip TEXT NOT NULL,
                         remote_port TEXT NOT NULL,                                                
                         status TEXT NOT NULL,
                         pid TEXT NOT NULL,
                         process_name TEXT NOT NULL,
                         today_count INT NOT NULL,
                         threat_score TEXT NOT NULL,
                         last_active TEXT NOT NULL,   
                         location TEXT NOT NULL   
                        
                         );''')
        print "Database connected"
        return db_conn
    except Error as e:
        print e
        print "Error"
        
    

def print_database(records):
    template="{:<20}| {:>15}|{:>6} |{:>15}|{:>6} | {:<6} |{:<6} |{:<7}|{:<15}|{:<}"
    print template.format("Recent"," Local","Port", "Foreign", "Port", "PID","Threat",\
    "Count","Process","Location")  
    for i in records:
        local_ip=i[6]
        local_port=i[7]
        remote_ip=i[8]
        remote_port=i[9]
        p_id=i[11]
        p_name=i[12]
        loc_name=i[16]
        '''
        print("Recent= {:<20} Local= {:>15}:{:<6} Foreign= {:>15}:{:<6} PID= {:<6} Threat= {:<4} Count= {:<4} "\
                .format(str(i[2]),str(local_ip),str(local_port),str(remote_ip),str(remote_port),str(p_id),\
                str(i[-2]),str(i[-3])))'''
        print template.format(str(i[2]),str(local_ip),str(local_port),str(remote_ip),str(remote_port),str(p_id),\
                str(i[-2]),str(i[-3]),str(p_name),str(loc_name))
    return None

def record_exists(db_conn,ip=None,job=None, limit=1):
    if ip != None and job ==None:
        sql_query="SELECT * FROM pyort WHERE remote_ip=? ORDER BY id DESC  LIMIT ? "
        cursor=db_conn.execute(sql_query,(ip,limit))
        exist=cursor.fetchone()
        if exist is None:            
            return False, [0]*30 #return a zero array
        else:            
            return True, exist
    elif job !=None:
        if job == "count":
            sql_query="SELECT * FROM pyort WHERE DATE(first_time)=DATE('now') OR DATE(last_time)=DATE('now')  ORDER BY today_count DESC  LIMIT ? "
            cursor=db_conn.execute(sql_query,(limit,))
        elif job == "ip":
            sql_query="SELECT * FROM pyort WHERE remote_ip=? ORDER BY id DESC  LIMIT ? "
            cursor=db_conn.execute(sql_query,(ip,limit))
        exist=cursor.fetchall()
        if exist is None:            
            return False,[0]*30 #return a zero array
        else:            
            return True, exist
    else:
        False, [0]*30 #return a zero array   



import sys
import argparse
#sys.path.append("../pyort/pyort/")
#from pyort_fun import *
from .pyort_fun import *



def main():
    parser = argparse.ArgumentParser()
    #monitoring and logging
    parser.add_argument('-s','--start',action='store_true', help="Start monitoring of foregin IP's")
    parser.add_argument('-k','--kind',type=str,help="Similar to [kind] parameter in psutil.net_connections")
    parser.add_argument('-Sv','--save',action='store_true',help="Saving output in database")
    parser.add_argument('-x','--silent',action='store_true',help="Silent mode, will not print any output")
    #for viewing database
    parser.add_argument('-d','--database',action='store_true',help="Fetch recent rows from database")
    parser.add_argument('-o','--order',nargs='?',type=str,default='count',help="Fetch by [ip] or [count]")
    parser.add_argument('-c','--constant',type=str,help="Add ip details here")
    parser.add_argument('-l','--limit',nargs='?',type=int,default=10,help="Fetch rows from database")
    #version
    parser.add_argument('-v','--version',action='store_true',help="Print program version and exit")
    args = parser.parse_args()
    sys.stdout.write(str(pyort_start(args)))
    
def pyort_start(args):

    #config file and location
    configfile_name = "config.ini"
    directory=os.path.expanduser("~")+"/.config/pyort/"     
   
    #fetch values from config file
    db_path,db_name,time_interval,kd,hp_key,threat_update,VERSION,geo_ip=config_para(directory,configfile_name)

   
   
    
    if args.start == True or args.database == True:
        #connecting to database
        db_conn=sqlite_conn(db_path,db_name)
        
    if args.kind != None: 
        #kind argument from command line. uses the same parameter in psutil.net_connections
        kd=args.kind
    
    #validating input values and fetches rows from database
    if args.database == False and args.start ==False and args.version==False:
        print ("Warning: please specify -d or -s or -v.") 
        exit
    elif args.version == True and args.database == False and args.start == False:
        #print version and exit
        print(VERSION)
        exit
    elif args.database == True and args.start == True:
        print "Warning: -d and -s parameters are not allowed together."
        exit     
    elif args.database == True and args.start ==False:               
        if args.order==None:
            #default order is count
            args.order="count"
        if args.limit ==None:
            #default limit is 10
            limit=10
        else:
            limit=args.limit
        if args.order=="ip":
            if args.constant==None:
                print ("Please provide the [ip] in -c.")
            else:
                print("\nRecent records selected according to the foregin IP.\n")
                _,records=record_exists(db_conn,ip=args.constant,job="ip",limit=limit)
                print_database(records)
        elif args.order=="count":            
            print("\nToday's records selected according to the count of incidence.\n")
            _,records=record_exists(db_conn,job="count",limit=limit)
            print_database(records)
           
         
    #starts monitioring
    if args.start==True and args.database == False:
        
         #GeoIP location data download from maxmind
        if geo_ip !='' and geo_ip in ['True','true', 'Yes','yes','Y', 'y', '1']:
            geolite2_download(directory)        
        elif geo_ip !=''  and geo_ip not in ['True','true', 'Yes','yes','Y', 'y', '1']:
            print("\n Warning: Please add Yes/Y/True to [geo_ip] in [config.ini] file to enble GeoIP")
            print(" or \n To disable keept [geo_ip] empty \n")
            
        print("\nMonitoring "+kd+" connections.\n")  
        #Loop till exit
        #Print format
        template="{:<20}| {:>15}|{:>6} |{:>15}|{:>6} | {:<6} |{:<6} |{:<7}|{:<15}|{:<}"
        print template.format("Recent"," Local","Port", "Foreign", "Port", "PID","Threat","Count","Process","Location") 
        while True:
            count=0
            conn=psutil.net_connections(kind=kd)
                    
            for c in conn:
                fd= c[0]
                family_code=c[1]
                type_code=c[2]
                local_ip=extract_ip(c[3])
                local_port=extract_ip(c[3],False)
                remote_ip=extract_ip(c[4])
                remote_port=extract_ip(c[4],False)
                status_code=c[5]
                p_id=c[6]  
                p_name= get_process_name(p_id)            
                #if not an ip or a private ip then escape the loop
                if remote_ip==None or ipaddress.ip_address(unicode(remote_ip)).is_private==True:
                    continue
                           
                #verfiy if the ip exists in the database
                is_record_exists, t_count=record_exists(db_conn,remote_ip)

                #GeoIP location
                if geo_ip !=''  and geo_ip in ['True','true', 'Yes','yes','Y', 'y', '1']:
                    loc_name =geoip2_location(directory,remote_ip)       
                   
                else:
                    loc_name=None
                    
                #updating project_honey_pot threat_score
                if hp_key!='' and int(count)%int(threat_update)==0 and args.save==True:
                    threat_score,last_active=project_honey_pot(remote_ip,hp_key)
                else:
                    threat_score,last_active=None,None
                
                if args.save==True:                    
                    if is_record_exists==False:                    
                        sql_query="""INSERT INTO pyort(fd,family,
                                           conn_type,local_ip,local_port,remote_ip,remote_port,
                                           status,pid,process_name,today_count,threat_score,last_active,location)
                                           VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?)"""

                        db_conn.execute(sql_query,(fd, family_code, type_code,str(local_ip),str(local_port),\
                                                   str(remote_ip),str(remote_port),status_code,str(p_id),\
                                                    str(p_name),count,str(threat_score),str(last_active),str(loc_name)))
                       
                    else:            
                        sql_query="""UPDATE pyort SET last_time=DATETIME('now'),
                             today_count=today_count+1,threat_score=?,last_active=?,pid=?,process_name=? where remote_ip=?"""
                        db_conn.execute(sql_query,(str(threat_score),str(last_active),str(p_id),str(p_name),remote_ip))
                if args.silent!=True: 
                    '''
                    if is_record_exists==False:
                        #suppressing nonetype error
                        tcount=[0]*10                 
                    print("Recent= {:<20} Local= {:>15}:{:<6} Foreign= {:>15}:{:<6} PID= {:<6} Threat= {:<4} Count= {:<4} "\
                    .format(str(t_count[2]),str(local_ip),str(local_port),str(remote_ip),\
                     str(remote_port),str(p_id),str(t_count[-2]),str(t_count[-3])))                 
                     '''
                    print template.format(str(t_count[2]),str(local_ip),str(local_port),str(remote_ip),\
                                          str(remote_port),str(p_id),str(t_count[14]),str(t_count[13]),\
                                          str(p_name),str(loc_name))

            db_conn.commit()

            time.sleep(float(time_interval))

        

if __name__=='__main__':     
    main()










from setuptools import setup

setup(
    # Needed to silence warnings (and to be a worthwhile package)
    name='pyort',
    url='https://github.com/Gananath/Pyort',
    author='Gananath R',
    author_email='nomail@nomail.com',
    # Needed to actually package something
    packages=['pyort'],
    # Needed for dependencies
    install_requires=['psutil','argparse','pysqlite'],
    # *strongly* suggested for sharing
    version='0.1.7.4',
    
    entry_points = {
        'console_scripts': [
            'pyort=pyort.pyort:main',
        ],
    },
    # The license can be anything you like
    license='MIT',
    description='Command line tool for monitoring all network connections',
    # We will also need a readme eventually (there will be a warning)
    # long_description=open('README.txt').read(),
)
